@@ -15,11 +15,17 @@
 //! Cloud API.
 
 #[allow(unused_imports)]
-use super::Result;
+use fallible_iterator::FallibleIterator;
+
+#[allow(unused_imports)]
+use super::{Error, ErrorKind, Result};
 use super::auth::AuthMethod;
 #[cfg(feature = "compute")]
-use super::compute::{Server, ServerQuery, ServerSummary};
+use super::compute::{Flavor, FlavorQuery, FlavorSummary,
+                     Server, ServerQuery, ServerSummary};
 use super::session::Session;
+#[allow(unused_imports)]
+use super::utils::ResultExt;
 
 
 /// OpenStack cloud API.
@@ -86,7 +92,7 @@ impl Cloud {
         self.session
     }
 
-    /// Find a server by its ID.
+    /// Find a server by its name or ID.
     ///
     /// # Example
     ///
@@ -95,19 +101,35 @@ impl Cloud {
     ///
     /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
     /// let os = openstack::Cloud::new(auth);
-    /// let server = os.get_server_by_id("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
+    /// let server = os.get_server("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
     ///     .expect("Unable to get a server");
     /// ```
     #[cfg(feature = "compute")]
-    pub fn get_server_by_id<Id: AsRef<str>>(&self, id: Id) -> Result<Server> {
-        Server::new(&self.session, id)
+    pub fn get_server<Id: Into<String>>(&self, id_or_name: Id) -> Result<Server> {
+        let s = id_or_name.into();
+        Server::new(&self.session, &s).if_not_found_then(|| {
+            self.find_servers().with_name(s.clone()).into_iter()
+                .filter(|srv| srv.name() == &s).take(2)
+                .collect::<Vec<ServerSummary>>().and_then(|mut srvs| {
+                    if srvs.len() > 1 {
+                        Err(Error::new(ErrorKind::TooManyItems,
+                                       "Too many servers with this name"))
+                    } else {
+                        match srvs.pop() {
+                            Some(srv) => srv.details(),
+                            None => Err(Error::new(
+                                ErrorKind::ResourceNotFound,
+                                "No servers with this name or ID"))
+                        }
+                    }
+                })
+        })
     }
 
     /// Build a query against server list.
     ///
     /// The returned object is a builder that should be used to construct
-    /// the query. The results can be received with a
-    /// [fetch](compute/struct.ServerQuery.html#method.fetch) call.
+    /// the query.
     ///
     /// # Example
     ///
@@ -121,7 +143,7 @@ impl Cloud {
     /// let sorting = openstack::compute::ServerSortKey::AccessIpv4;
     /// let server_list = os.find_servers()
     ///     .sort_by(openstack::Sort::Asc(sorting)).with_limit(5)
-    ///     .fetch().expect("Unable to fetch servers");
+    ///     .all().expect("Unable to fetch servers");
     /// ```
     #[cfg(feature = "compute")]
     pub fn find_servers(&self) -> ServerQuery {
@@ -145,8 +167,69 @@ impl Cloud {
     /// ```
     #[cfg(feature = "compute")]
     pub fn list_servers(&self) -> Result<Vec<ServerSummary>> {
-        // TODO(dtantsur): pagination
-        self.find_servers().fetch()
+        self.find_servers().all()
+    }
+
+    /// Find a flavor by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server = os.get_flavor("m1.medium").expect("Unable to get a flavor");
+    /// ```
+    #[cfg(feature = "compute")]
+    pub fn get_flavor<Id: Into<String>>(&self, id_or_name: Id) -> Result<Flavor> {
+        let s = id_or_name.into();
+        Flavor::new(&self.session, &s).if_not_found_then(|| {
+            self.find_flavors().into_iter()
+                .filter(|item| item.name() == &s).take(2)
+                .collect::<Vec<FlavorSummary>>().and_then(|mut items| {
+                    if items.len() > 1 {
+                        Err(Error::new(ErrorKind::TooManyItems,
+                                       "Too many flavors with this name"))
+                    } else {
+                        match items.pop() {
+                            Some(item) => item.details(),
+                            None => Err(Error::new(
+                                ErrorKind::ResourceNotFound,
+                                "No flavors with this name or ID"))
+                        }
+                    }
+                })
+        })
+    }
+
+    /// Build a query against flavor list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "compute")]
+    pub fn find_flavors(&self) -> FlavorQuery {
+        FlavorQuery::new(&self.session)
+    }
+
+    /// List all flavors.
+    ///
+    /// This call can yield a lot of results, use the
+    /// [find_flavors](#method.find_flavors) call to limit the number of
+    /// flavors to receive.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
+    /// let os = openstack::Cloud::new(auth);
+    /// let server_list = os.list_flavors().expect("Unable to fetch flavors");
+    /// ```
+    #[cfg(feature = "compute")]
+    pub fn list_flavors(&self) -> Result<Vec<FlavorSummary>> {
+        self.find_flavors().all()
     }
 }
 
