@@ -22,15 +22,18 @@
 //! let auth = openstack::auth::from_env().expect("Unable to authenticate");
 //! let os = openstack::Cloud::new(auth);
 //!
-//! let server = os.get_server_by_id("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
+//! let server = os.get_server("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
 //!     .expect("Unable to get a server");
 //! ```
 
+mod base;
+mod flavors;
+mod protocol;
 mod servers;
-mod v2;
 
-pub use self::v2::V2 as ServiceType;
-pub use self::v2::protocol::{AddressType, ServerAddress, ServerSortKey,
-                             ServerStatus};
-pub use self::servers::{Server, ServerList, ServerQuery, ServerSummary,
-                        FlavorRef, ImageRef};
+pub use self::base::V2 as ServiceType;
+pub use self::flavors::{Flavor, FlavorSummary, FlavorQuery, ToFlavorId};
+pub use self::protocol::{AddressType, ServerAddress, ServerSortKey,
+                         ServerPowerState, ServerStatus};
+pub use self::servers::{Server, ServerQuery, ServerStatusWaiter, ServerSummary,
+                        ToImageId};
