@@ -62,8 +62,10 @@
         unused_results,
         while_true)]
 
-#[cfg(feature = "compute")]
+#[allow(unused_extern_crates)]
 extern crate chrono;
+#[allow(unused_extern_crates)]
+extern crate fallible_iterator;
 #[macro_use]
 extern crate log;
 extern crate reqwest;
@@ -85,7 +87,9 @@ mod utils;
 
 pub use cloud::Cloud;
 pub use common::Error;
+pub use common::ErrorKind;
 pub use common::Result;
 pub use common::ApiVersion;
 pub use common::ApiVersionRequest;
 pub use common::Sort;
+pub use common::Waiter;
