@@ -18,14 +18,13 @@
 
 use std::cell::{Ref, RefCell};
 use std::collections::HashMap;
-use std::fmt::Display;
 use std::hash::Hash;
-use std::str::FromStr;
 
 use serde::{Deserialize, Deserializer};
-use serde::de::Error as DeserError;
+use serde::de::{DeserializeOwned, Error as DeserError};
+use serde_json;
 
-use super::Result;
+use super::{ErrorKind, Result};
 
 
 /// Type of query parameters.
@@ -117,12 +116,49 @@ impl<K: Hash + Eq, V: Clone> MapCache<K, V> {
 
 /// Deserialize value where empty string equals None.
 pub fn empty_as_none<'de, D, T>(des: D) -> ::std::result::Result<Option<T>, D::Error>
-        where D: Deserializer<'de>, T: FromStr, T::Err: Display {
-    let s = String::deserialize(des)?;
-    if s.is_empty() {
-        Ok(None)
-    } else {
-        T::from_str(&s).map(Some).map_err(DeserError::custom)
+        where D: Deserializer<'de>, T: DeserializeOwned {
+    let value = serde_json::Value::deserialize(des)?;
+    match &value {
+        &serde_json::Value::String(ref s) if s == "" => return Ok(None),
+        _ => ()
+    };
+
+    match serde_json::from_value(value) {
+        Ok(value) => Ok(Some(value)),
+        Err(e) => Err(DeserError::custom(e))
+    }
+}
+
+/// Deserialize value where empty string equals None.
+pub fn empty_as_default<'de, D, T>(des: D) -> ::std::result::Result<T, D::Error>
+        where D: Deserializer<'de>, T: DeserializeOwned + Default {
+    let value = serde_json::Value::deserialize(des)?;
+    match &value {
+        &serde_json::Value::String(ref s) if s == "" =>
+            return Ok(Default::default()),
+        _ => ()
+    };
+
+    serde_json::from_value(value).map_err(DeserError::custom)
+}
+
+/// Extensions for Result type.
+pub trait ResultExt<T> {
+    /// Process result if the error was ResourceNotFound.
+    fn if_not_found_then<F>(self, f: F) -> Result<T>
+        where F: FnOnce() -> Result<T>;
+}
+
+impl<T> ResultExt<T> for Result<T> {
+    fn if_not_found_then<F>(self, f: F) -> Result<T>
+            where F: FnOnce() -> Result<T> {
+        self.or_else(|err| {
+            if err.kind() == ErrorKind::ResourceNotFound {
+                f()
+            } else {
+                Err(err)
+            }
+        })
     }
 }
 
@@ -172,7 +208,7 @@ pub mod test {
     use reqwest::{IntoUrl, Url};
     use reqwest::header::Headers;
 
-    use super::super::{Error, Result, ApiVersion};
+    use super::super::{Error, ErrorKind, Result, ApiVersion};
     use super::super::auth::{AuthMethod, NoAuth};
     use super::super::service::{ServiceInfo, ServiceType};
     use super::super::session::Session;
@@ -199,7 +235,7 @@ pub mod test {
                     minimum_version: Some(ApiVersion(1, 1)),
                 })
             } else {
-                Err(Error::EndpointNotFound(String::new()))
+                Err(Error::new(ErrorKind::EndpointNotFound, String::new()))
             }
         }
 
